@@ -11,6 +11,7 @@
 #include <string.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
+#include <ctype.h>
 #include "qiv.h"
 
 #define STEP 3 //When using KP arrow, number of step for seeing all the image.
@@ -60,34 +61,33 @@ static void qiv_cancel_cursor_timeout(qiv_image *q)
   }
 }
 
-static void qiv_drag_image(qiv_image *q, int move_to_x, int move_to_y)
+static void qiv_drag_image(qiv_image *q, int move_to_x, int move_to_y,
+                           const char *action_msg, const char *stalled_msg)
 {
-  q->win_x = move_to_x;
-  if (q->win_w > screen_x) {
-    if (q->win_x > 0)
-      q->win_x = 0;
-    else if (q->win_x + q->win_w < screen_x)
-      q->win_x = screen_x - q->win_w;
-  } else {
-    if (q->win_x < 0)
-      q->win_x = 0;
-    else if (q->win_x + q->win_w > screen_x)
-      q->win_x = screen_x - q->win_w;
+  int moved = 0;
+
+  if (move_to_x < 0 - q->win_w)
+    move_to_x = 0 - q->win_w;
+  else if (move_to_x > monitor[q->mon_id].width)
+    move_to_x = monitor[q->mon_id].width;
+  if (q->win_x != move_to_x) {
+    q->win_x = move_to_x;
+    moved = 1;
   }
 
-  q->win_y = move_to_y;
-  if (q->win_h > screen_y) {
-    if (q->win_y > 0)
-      q->win_y = 0;
-    else if (q->win_y + q->win_h < screen_y)
-      q->win_y = screen_y - q->win_h;
-  } else {
-    if (q->win_y < 0)
-      q->win_y = 0;
-    else if (q->win_y + q->win_h > screen_y)
-      q->win_y = screen_y - q->win_h;
+  if (move_to_y < 0 - q->win_h)
+    move_to_y = 0 - q->win_h;
+  else if (move_to_y > monitor[q->mon_id].height)
+    move_to_y = monitor[q->mon_id].height;
+  if (q->win_y != move_to_y) {
+    q->win_y = move_to_y;
+    moved = 1;
   }
 
+  if (moved || !stalled_msg)
+    snprintf(infotext, sizeof infotext, "%s", action_msg);
+  else
+    snprintf(infotext, sizeof infotext, "%s", stalled_msg);
   update_image(q, MOVED);
 }
 
@@ -95,7 +95,7 @@ void qiv_display_text_window(qiv_image *q, const char *infotextdisplay,
                              const char *strs[], const char *continue_msg)
 {
   int temp, text_w = 0, text_h, i, maxlines;
-  int width, height, text_left;
+  int x, y, width, height, text_left;
 
   int ascent;
   int descent;
@@ -104,9 +104,11 @@ void qiv_display_text_window(qiv_image *q, const char *infotextdisplay,
   descent = PANGO_PIXELS(pango_font_metrics_get_descent(metrics));
 
   if (fullscreen) {
-    width = screen_x;
-    height = screen_y;
+    x = y = 0;
+    width = monitor[q->mon_id].width;
+    height = monitor[q->mon_id].height;
   } else {
+    x = y = 0;
     width = q->win_w;
     height = q->win_h;
   }
@@ -128,22 +130,22 @@ void qiv_display_text_window(qiv_image *q, const char *infotextdisplay,
   text_h = (i + 2) * ( ascent + descent );
 
   snprintf(infotext, sizeof infotext, "%s", infotextdisplay);
-  update_image(q, REDRAW);
+  update_image(q, MIN_REDRAW);
 
   text_left = width/2 - text_w/2 - 4;
   if (text_left < 2)  text_left = 2;            /* if window/screen is smaller than text */
 
-  gdk_draw_rectangle(q->win, q->bg_gc, 0,
-                     text_left,
-                     height/2 - text_h/2 - 4,
-                     text_w + 7, text_h + 7);
-  gdk_draw_rectangle(q->win, q->status_gc, 1,
-                     text_left + 1,
-                     height/2 - text_h/2 - 3,
-                     text_w + 6, text_h + 6);
+  gdk_draw_rectangle(q->win, q->bg_gc, y,
+                     x + text_left,
+                     y + height/2 - text_h/2 - 4,
+                     x + text_w + 7, text_h + 7);
+  gdk_draw_rectangle(q->win, q->status_gc, y + 1,
+                     x + text_left + 1,
+                     y + height/2 - text_h/2 - 3,
+                     x + text_w + 6, text_h + 6);
   for (i = 0; strs[i] && i < maxlines; i++) {
        pango_layout_set_text(layout, strs[i], -1);
-       gdk_draw_layout (q->win, q->text_gc, text_left + 4, height/2 - text_h/2  +
+       gdk_draw_layout (q->win, q->text_gc, x + text_left + 4, y + height/2 - text_h/2  +
                   i * (ascent + descent), layout);
   }
 
@@ -151,8 +153,8 @@ void qiv_display_text_window(qiv_image *q, const char *infotextdisplay,
   pango_layout_set_text(layout, continue_msg, -1);
   pango_layout_get_pixel_size (layout, &temp, NULL);
   gdk_draw_layout (q->win, q->text_gc, 
-                   width/2 - temp/2,
-                   height/2 - text_h/2 - descent + (i+1) * (ascent + descent),
+                   x + width/2 - temp/2,
+                   y + height/2 - text_h/2 - descent + (i+1) * (ascent + descent),
                    layout);
   displaying_textwindow = TRUE;
 
@@ -177,10 +179,9 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
   // (because the user might have moved the window our since last redraw)
   if (!fullscreen) {
     gdk_window_get_position(q->win, &q->win_x, &q->win_y);
-//    g_print("position   : q->win_x = %d, q->win_y = %d, q->win_w = %d\n", q->win_x, q->win_y, q->win_w);
-//    gdk_window_get_origin(q->win, &q->win_x, &q->win_y);
-//    gdk_window_get_root_origin(q->win, &q->win_x, &q->win_y);
+//    g_print("position   : q->win_x = %d, q->win_y = %d, q->win_w = %d, mon = %d\n", q->win_x, q->win_y, q->win_w,q->mon_id);
   }
+  q->mon_id = gdk_screen_get_monitor_at_window(screen, q->win);
 
   switch(ev->type) {
     case GDK_DELETE:
@@ -192,14 +193,29 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
         q->exposed = 1;
         qiv_set_cursor_timeout(q);
       }
+      if(fullscreen) {
+        if(center) center_image(q);
+        update_image(q, FULL_REDRAW);
+      }
       break;
 
     case GDK_LEAVE_NOTIFY:
       if (magnify && !fullscreen) {
         gdk_window_hide(magnify_img.win);
       }
+      gdk_pointer_ungrab(CurrentTime);
+      gdk_keyboard_ungrab(CurrentTime);
       break;
 
+    case GDK_ENTER_NOTIFY:
+      if(do_grab || (fullscreen && !disable_grab))
+      {
+        gdk_keyboard_grab(q->win, FALSE, CurrentTime);
+        gdk_pointer_grab(q->win, FALSE,
+            GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK |
+            GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK, NULL, NULL, CurrentTime);
+      }
+
     case GDK_CONFIGURE:
       if (magnify && !fullscreen) {
         gdk_window_get_root_origin(q->win,
@@ -208,7 +224,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
         //        magnify_img.frame_x, magnify_img.frame_y);
       }
       // gdk_draw_rectangle(q->win, q->status_gc, 1, 10, 10, 50, 50);
-      if (statusbar_window) {
+      if (statusbar_window && !fullscreen) {
 #ifdef DEBUG
         g_print("*** print statusbar at (%d, %d)\n", MAX(2,q->win_w-q->text_w-10), MAX(2,q->win_h-q->text_h-10));
 #endif
@@ -234,8 +250,10 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
       qiv_cancel_cursor_timeout(q);
       if (fullscreen && ev->button.button == 1) {
         q->drag = 1;
-        q->drag_start_x = ev->button.x;
-        q->drag_start_y = ev->button.y;
+//        q->drag_start_x = ev->button.x;
+//        q->drag_start_y = ev->button.y;
+        q->drag_start_x = ev->button.x_root;
+        q->drag_start_y = ev->button.y_root;
         q->drag_win_x = q->win_x;
         q->drag_win_y = q->win_y;
         qiv_enable_mouse_events(q);
@@ -245,8 +263,8 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
     case GDK_MOTION_NOTIFY:
       if (q->drag) {
         int move_x, move_y;
-        move_x = (int)(ev->button.x - q->drag_start_x);
-        move_y = (int)(ev->button.y - q->drag_start_y);
+        move_x = (int)(ev->button.x_root - q->drag_start_x);
+        move_y = (int)(ev->button.y_root - q->drag_start_y);
         if (q->drag == 1 && (ABS(move_x) > 3 || ABS(move_y) > 3)) {
           /* distinguish from simple mouse click... */
           q->drag = 2;
@@ -256,17 +274,18 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
                             q->win_y != q->drag_win_y + move_y)) {
           GdkEvent *e;
           qiv_disable_mouse_events(q);
-          qiv_drag_image(q, q->drag_win_x + move_x, q->drag_win_y + move_y);
-          snprintf(infotext, sizeof infotext, "(Drag)");
+          qiv_drag_image(q, q->drag_win_x + move_x, q->drag_win_y + move_y, "(Drag)", NULL);
           /* el cheapo mouse motion compression */
           while (gdk_events_pending()) {
-            e = gdk_event_get();
-            if (e->type == GDK_BUTTON_RELEASE) {
-              gdk_event_put(e);
+            /* [tw] why check if "e" not NULL? */
+            if((e = gdk_event_get())) {
+              if (e->type == GDK_BUTTON_RELEASE) {
+                gdk_event_put(e);
+                gdk_event_free(e);
+                break;
+              }
               gdk_event_free(e);
-              break;
             }
-            gdk_event_free(e);
           }
           qiv_enable_mouse_events(q);
         }
@@ -295,26 +314,31 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
         /* [tw]: I think buttons 4-7 don't work in GDK-2.0 as button events,
          *       they are now GDK_SCROLL events */
         case 1:        /* left button pressed */
+          displaying_textwindow = FALSE;
           if (q->drag) {
             int move_x, move_y;
-            move_x = (int)(ev->button.x - q->drag_start_x);
-            move_y = (int)(ev->button.y - q->drag_start_y);
+//            move_x = (int)(ev->button.x - q->drag_start_x);
+//            move_y = (int)(ev->button.y - q->drag_start_y);
+            move_x = (int)(ev->button.x_root - q->drag_start_x);
+            move_y = (int)(ev->button.y_root - q->drag_start_y);
             qiv_disable_mouse_events(q);
             qiv_set_cursor_timeout(q);
 
             if (q->drag > 1) {
-              qiv_drag_image(q, q->drag_win_x + move_x, q->drag_win_y + move_y);
-              snprintf(infotext, sizeof infotext, "(Drag)");
+              qiv_drag_image(q, q->drag_win_x + move_x, q->drag_win_y + move_y, "(Drag)", NULL);
               q->drag = 0;
               break;
             }
             q->drag = 0;
           }
         case 5:        /* scroll wheel down emulated by button 5 */
+          displaying_textwindow = FALSE;
           goto next_image;
         case 8:
+          displaying_textwindow = FALSE;
           goto zoom_out;
         case 9:
+          displaying_textwindow = FALSE;
           goto zoom_in;
         default:
           g_print("unmapped button event %d, exiting\n",ev->button.button);
@@ -323,6 +347,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
           break;
         case 3:        /* right button pressed */
         case 4:        /* scroll wheel up emulated by button 4 */
+          displaying_textwindow = FALSE;
           goto previous_image;
           break;
       }
@@ -331,12 +356,16 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
     case GDK_SCROLL:
       switch (ev->scroll.direction) {
         case GDK_SCROLL_UP:
-	  goto next_image;
-        case GDK_SCROLL_DOWN:
+          displaying_textwindow = FALSE;
 	  goto previous_image;
+        case GDK_SCROLL_DOWN:
+          displaying_textwindow = FALSE;
+	  goto next_image;
         case GDK_SCROLL_LEFT:
+          displaying_textwindow = FALSE;
           goto zoom_out;
         case GDK_SCROLL_RIGHT:
+          displaying_textwindow = FALSE;
           goto zoom_in;
         default:
 	  break;
@@ -356,27 +385,35 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
       if (displaying_textwindow && !extcommand ) { // [lc]
         /* Hide the text window if it is showing */
         displaying_textwindow = FALSE;
-        update_image(q, FULL_REDRAW);
+        if(fullscreen)
+        {
+          update_image(q, FULL_REDRAW);
+        }
+        else
+        {
+          update_image(q, MIN_REDRAW);
+        }
+
         break;
       }
 
       if (jumping || extcommand) {  // [lc] todo: Backspace,
-        if(ev->key.keyval == GDK_Escape) {
+        if(ev->key.keyval == GDK_KEY_Escape) {
           extcommand = 0;
           jidx = 0;
           displaying_textwindow = FALSE;
           update_image(q, FULL_REDRAW);
           break;
         }
-        if(ev->key.keyval == GDK_BackSpace) {
+        if(ev->key.keyval == GDK_KEY_BackSpace) {
           jidx--;
           jcmd[jidx]='\0';
           qiv_display_text_window(q, "(Sending Command)", mess,
                                   "Press <Return> to send, <Esc> to abort"); // [lc]
           break;
         }
-        if((ev->key.keyval == GDK_Return) ||
-           (ev->key.keyval == GDK_KP_Enter) ||
+        if((ev->key.keyval == GDK_KEY_Return) ||
+           (ev->key.keyval == GDK_KEY_KP_Enter) ||
            (jidx == 99)) {     /* 99 digits already typed */
           jcmd[jidx] = '\0';
           if (jumping) {
@@ -391,7 +428,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             extcommand=0;
             /* Hide the text window if it is showing */
             displaying_textwindow = FALSE;
-            update_image(q, FULL_REDRAW);
+            update_image(q, MIN_REDRAW);
             run_command(q, jcmd, image_names[image_idx], &numlines, &lines);
             if (lines && numlines)
               qiv_display_text_window(q, "(Command output)", lines, "Push any key...");
@@ -406,18 +443,34 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
                                     "Press <Return> to send, <Esc> to abort"); // [lc]
         }
       } else {
-        switch (ev->key.keyval) {
+        guint keyval = ev->key.keyval;
+        if(vikeys) {
+               if (keyval == 'h') keyval = GDK_Left;
+          else if (keyval == 'j') keyval = GDK_Down;
+          else if (keyval == 'k') keyval = GDK_Up;
+          else if (keyval == 'l') keyval = GDK_Right;
+          else
+            switch (keyval) {
+              case 'H':
+              case 'J':
+              case 'K':
+              case 'L':
+                keyval = tolower(keyval);
+                break;
+            }
+        }
+        switch (keyval) {
 
           /* Help */
 
           case '?':
-          case GDK_F1:
+          case GDK_KEY_F1:
             qiv_display_text_window(q, "(Showing Help)", helpstrs,"Press any key...");
             break;
 
             /* Exit */
 
-          case GDK_Escape:
+          case GDK_KEY_Escape:
           case 'q':
             qiv_exit(0);
             break;
@@ -444,6 +497,27 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             update_image(q, MOVED);
             break;
 
+            /* Display exif information */
+#ifdef HAVE_EXIF
+          case 'E':
+            {
+              char **lines;
+              int i=0;
+              lines=get_exif_values(image_names[image_idx]);
+              if(lines)
+              {
+                qiv_display_text_window(q, "(Exif Info)", (const char**) lines,"Press any key...");
+                /* free all the mem malloc'ed by get_exif_values func */
+                while(*(lines+i) != NULL)
+                {
+                  free(*(lines+i));
+                  i++;
+                }
+                free(lines);
+              }
+            }
+            break;
+#endif
             /* Transparency on/off */
 
           case 'p':
@@ -451,7 +525,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             transparency ^= 1;
             snprintf(infotext, sizeof infotext, transparency ?
                      "(Transparency: on)" : "(Transparency: off)");
-            update_image(q, FULL_REDRAW);
+            update_image(q, MIN_REDRAW);
             break;
 
             /* Maxpect on/off */
@@ -504,7 +578,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
               snprintf(infotext, sizeof infotext, statusbar_window ?
                        "(Statusbar: on)" : "(Statusbar: off)");
             }
-            update_image(q, REDRAW);
+            update_image(q, MIN_REDRAW);
             break;
 
             /* Slide show on/off */
@@ -512,197 +586,97 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
           case 's':
             exit_slideshow = FALSE;
             slide ^= 1;
+            if (slide) 
+            {
+              dpms_disable();
+            }
             snprintf(infotext, sizeof infotext, slide ?
                      "(Slideshow: on)" : "(Slideshow: off)");
-            update_image(q, REDRAW);
+            update_image(q, MIN_REDRAW);
             break;
 
             /* move image right */
 
-          case GDK_Left:
-          case GDK_KP_4:
-          case GDK_KP_Left:
+          case GDK_KEY_Left:
+          case GDK_KEY_KP_4:
+          case GDK_KEY_KP_Left:
             if (fullscreen) {
-              if (ev->key.state & GDK_SHIFT_MASK || ev->key.keyval == GDK_KP_4) {
-                move_step = (MIN(screen_x, q->win_w) / STEP);
+              if (ev->key.state & GDK_SHIFT_MASK || ev->key.keyval == GDK_KEY_KP_4) {
+                move_step = (MIN(monitor[q->mon_id].width, q->win_w) / STEP);
               } else {
                 move_step = (q->win_w / 100);
               }
               if (move_step < 10)
                 move_step = 10;
 
-              /* is image greater than screen? */
-              if (q->win_w > screen_x) {
-                /* left border visible yet? */
-                if (q->win_x < 0) {
-                  q->win_x += move_step;
-                  /* sanity check */
-                  if (q->win_x > 0)
-                    q->win_x = 0;
-                  snprintf(infotext, sizeof infotext, "(Moving right)");
-                } else {
-                  snprintf(infotext, sizeof infotext, "(Cannot move further to the right)");
-                }
-
-              } else {                      /* user is just playing around */
-
-                /* right border reached? */
-                if (q->win_x + q->win_w < screen_x) {
-                  q->win_x += move_step;
-                  /* sanity check */
-                  if (q->win_x + q->win_w > screen_x)
-                    q->win_x = screen_x - q->win_w;
-                  snprintf(infotext, sizeof infotext, "(Moving right)");
-                } else {
-                  snprintf(infotext, sizeof infotext, "(Cannot move further to the right)");
-                }
-              }
+              qiv_drag_image(q, q->win_x + move_step, q->win_y, "(Moving right)", "(Cannot move further right)");
             } else {
               snprintf(infotext, sizeof infotext, "(Moving works only in fullscreen mode)");
               fprintf(stdout, "qiv: Moving works only in fullscreen mode\n");
             }
-            update_image(q, MOVED);
             break;
 
             /* move image left */
 
-          case GDK_Right:
-          case GDK_KP_6:
-          case GDK_KP_Right:
+          case GDK_KEY_Right:
+          case GDK_KEY_KP_6:
+          case GDK_KEY_KP_Right:
             if (fullscreen) {
-              if (ev->key.state & GDK_SHIFT_MASK || ev->key.keyval == GDK_KP_6) {
-                move_step = (MIN(screen_x, q->win_w) / STEP);
+              if (ev->key.state & GDK_SHIFT_MASK || ev->key.keyval == GDK_KEY_KP_6) {
+                move_step = (MIN(monitor[q->mon_id].width, q->win_w) / STEP);
               } else {
                 move_step = (q->win_w / 100);
               }
               if (move_step < 10)
                 move_step = 10;
 
-              /* is image greater than screen? */
-              if (q->win_w > screen_x) {
-                /* right border visible yet? */
-                if (q->win_x + q->win_w > screen_x) {
-                  q->win_x -= move_step;
-                  /* sanity check */
-                  if (q->win_x + q->win_w < screen_x)
-                    q->win_x = screen_x - q->win_w;
-                  snprintf(infotext, sizeof infotext, "(Moving left)");
-                } else {
-                  snprintf(infotext, sizeof infotext, "(Cannot move further to the left)");
-                }
-
-              } else {                      /* user is just playing around */
-
-                /* left border reached? */
-                if (q->win_x > 0) {
-                  q->win_x -= move_step;
-                  /* sanity check */
-                  if (q->win_x < 0)
-                    q->win_x = 0;
-                  snprintf(infotext, sizeof infotext, "(Moving left)");
-                } else {
-                  snprintf(infotext, sizeof infotext, "(Cannot move further to the left)");
-                }
-              }
+              qiv_drag_image(q, q->win_x - move_step, q->win_y, "(Moving left)", "(Cannot move further left)");
             } else {
               snprintf(infotext, sizeof infotext, "(Moving works only in fullscreen mode)");
               fprintf(stdout, "qiv: Moving works only in fullscreen mode\n");
             }
-            update_image(q, MOVED);
             break;
 
             /* move image up */
 
-          case GDK_Down:
-          case GDK_KP_2:
-          case GDK_KP_Down:
+          case GDK_KEY_Down:
+          case GDK_KEY_KP_2:
+          case GDK_KEY_KP_Down:
             if (fullscreen) {
-              if (ev->key.state & GDK_SHIFT_MASK || ev->key.keyval == GDK_KP_2) {
-                move_step = (MIN(screen_y, q->win_h) / STEP);
+              if (ev->key.state & GDK_SHIFT_MASK || ev->key.keyval == GDK_KEY_KP_2) {
+                move_step = (MIN(monitor[q->mon_id].height, q->win_h) / STEP);
               } else {
                 move_step = (q->win_h / 100);
               }
               if (move_step < 10)
                 move_step = 10;
 
-              /* is image greater than screen? */
-              if (q->win_h > screen_y) {
-                /* bottom visible yet? */
-                if (q->win_y + q->win_h > screen_y) {
-                  q->win_y -= move_step;
-                  /* sanity check */
-                  if (q->win_y + q->win_h < screen_y)
-                    q->win_y = screen_y - q->win_h;
-                  snprintf(infotext, sizeof infotext, "(Moving up)");
-                } else {
-                  snprintf(infotext, sizeof infotext, "(Cannot move further up)");
-                }
-
-              } else {                      /* user is just playing around */
-
-                /* top reached? */
-                if (q->win_y > 0) {
-                  q->win_y -= move_step;
-                  /* sanity check */
-                  if (q->win_y < 0)
-                    q->win_y = 0;
-                  snprintf(infotext, sizeof infotext, "(Moving up)");
-                } else {
-                  snprintf(infotext, sizeof infotext, "(Cannot move further up)");
-                }
-              }
+              qiv_drag_image(q, q->win_x, q->win_y - move_step, "(Moving up)", "(Cannot move further up)");
             } else {
               snprintf(infotext, sizeof infotext, "(Moving works only in fullscreen mode)");
               fprintf(stdout, "qiv: Moving works only in fullscreen mode\n");
             }
-            update_image(q, MOVED);
             break;
 
             /* move image down */
 
-          case GDK_Up:
-          case GDK_KP_8:
-          case GDK_KP_Up:
+          case GDK_KEY_Up:
+          case GDK_KEY_KP_8:
+          case GDK_KEY_KP_Up:
             if (fullscreen) {
-              if (ev->key.state & GDK_SHIFT_MASK || ev->key.keyval == GDK_KP_8) {
-                move_step = (MIN(screen_y, q->win_h) / STEP);
+              if (ev->key.state & GDK_SHIFT_MASK || ev->key.keyval == GDK_KEY_KP_8) {
+                move_step = (MIN(monitor[q->mon_id].height, q->win_h) / STEP);
               } else {
                 move_step = (q->win_h / 100);
               }
               if (move_step < 10)
                 move_step = 10;
 
-              /* is image greater than screen? */
-              if (q->win_h > screen_y) {
-                /* top visible yet? */
-                if (q->win_y < 0) {
-                  q->win_y += move_step;
-                  /* sanity check */
-                  if (q->win_y > 0)
-                    q->win_y = 0;
-                  snprintf(infotext, sizeof infotext, "(Moving down)");
-                } else {
-                  snprintf(infotext, sizeof infotext, "(Cannot move further down)");
-                }
-
-              } else {                      /* user is just playing around */
-
-                /* bottom reached? */
-                if (q->win_y + q->win_h < screen_y) {
-                  q->win_y += move_step;
-                  /* sanity check */
-                  if (q->win_y + q->win_h > screen_y)
-                    q->win_y = screen_y - q->win_h;
-                  snprintf(infotext, sizeof infotext, "(Moving down)");
-                } else {
-                  snprintf(infotext, sizeof infotext, "(Cannot move further down)");
-                }
-              }
+              qiv_drag_image(q, q->win_x, q->win_y + move_step, "(Moving down)", "(Cannot move further down)");
             } else {
               snprintf(infotext, sizeof infotext, "(Moving works only in fullscreen mode)");
               fprintf(stdout, "qiv: Moving works only in fullscreen mode\n");
             }
-            update_image(q, MOVED);
             break;
 
             /* Scale_down */
@@ -719,29 +693,51 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
             /* Resize + */
 
-          case GDK_KP_Add:
+          case GDK_KEY_KP_Add:
           case '+':
           case '=':
           zoom_in:
-            snprintf(infotext, sizeof infotext, "(Zoomed in)");
-            zoom_in(q);
-            update_image(q, ZOOMED);
+            if (magnify && !fullscreen) {
+              gint xcur, ycur;
+              magnify_img.zoom *= 1.1;
+              gdk_window_get_pointer(q->win, &xcur, &ycur, NULL);
+              update_magnify(q, &magnify_img,REDRAW, xcur, ycur); // [lc]
+            }
+            else
+            {
+              snprintf(infotext, sizeof infotext, "(Zoomed in)");
+              zoom_in(q);
+              update_image(q, ZOOMED);
+            }
             break;
 
             /* Resize - */
 
-          case GDK_KP_Subtract:
+          case GDK_KEY_KP_Subtract:
           case '-':
           zoom_out:
-            snprintf(infotext, sizeof infotext, "(Zoomed out)");
-            zoom_out(q);
-            update_image(q, ZOOMED);
+            if (magnify && !fullscreen) {
+              if (magnify_img.zoom > 2.0)
+              {
+                gint xcur, ycur;
+                magnify_img.zoom /= 1.1;
+                if (magnify_img.zoom < 2) magnify_img.zoom = 2.0;
+                gdk_window_get_pointer(q->win, &xcur, &ycur, NULL);
+                update_magnify(q, &magnify_img,REDRAW, xcur, ycur);
+              }
+            }
+            else
+            {
+              snprintf(infotext, sizeof infotext, "(Zoomed out)");
+              zoom_out(q);
+              update_image(q, ZOOMED);
+            }
             break;
 
             /* Reset Image / Original (best fit) size */
 
-          case GDK_Return:
-          case GDK_KP_Enter:
+          case GDK_KEY_Return:
+          case GDK_KEY_KP_Enter:
             snprintf(infotext, sizeof infotext, "(Reset size)");
             reload_image(q);
             zoom_factor = fixed_zoom_factor;  /* reset zoom */
@@ -768,8 +764,8 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
             /* 5 pictures forward - or loop to the beginning */
 
-          case GDK_Page_Down:
-          case GDK_KP_Page_Down:
+          case GDK_KEY_Page_Down:
+          case GDK_KEY_KP_Page_Down:
             snprintf(infotext, sizeof infotext, "(5 pictures forward)");
             next_image(5);
             if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
@@ -778,7 +774,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
             /* Previous picture - or loop back to the last */
 
-          case GDK_BackSpace:
+          case GDK_KEY_BackSpace:
           previous_image:
             snprintf(infotext, sizeof infotext, "(Previous picture)");
             next_image(-1);
@@ -788,8 +784,8 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
             /* 5 pictures backward - or loop back to the last */
 
-          case GDK_Page_Up:
-          case GDK_KP_Page_Up:
+          case GDK_KEY_Page_Up:
+          case GDK_KEY_KP_Page_Up:
             snprintf(infotext, sizeof infotext, "(5 pictures backward)");
             next_image(-5);
             if(magnify && !fullscreen)    gdk_window_hide(magnify_img.win); // [lc]
@@ -854,7 +850,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
             /* Delete image */
 
-          case GDK_Delete:
+          case GDK_KEY_Delete:
           case 'd':
             if (!readonly) {
               if (move2trash() == 0)
@@ -977,7 +973,7 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
 
 
             /* Decrease slideshow delay */
-          case GDK_F11:
+          case GDK_KEY_F11:
             exit_slideshow = FALSE;
             if (delay > 1000) {
               delay-=1000;
@@ -1006,38 +1002,44 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
             break;
 
             /* Increase slideshow delay */
-          case GDK_F12:
+          case GDK_KEY_F12:
             exit_slideshow = FALSE;
             delay+=1000;
             snprintf(infotext, sizeof infotext, "(Slideshow-Delay: %d seconds (+1)", delay/1000);
             update_image(q,MOVED);
             break;
 
-#ifdef GTD_XINERAMA
-            /* go to next xinerama screen */
           case 'X':
-            if (number_xinerama_screens) {
-              user_screen++;
-              user_screen %= number_xinerama_screens;
-              // g_print("user_screen = %d, number_xinerama_screens = %d\n", user_screen, number_xinerama_screens);
+            if(num_monitors > 1) {
+              q->mon_id = (q->mon_id + 1) % num_monitors;
+              qiv_load_image(q);
             }
-            get_preferred_xinerama_screens();     // reselect appropriate screen
-            snprintf(infotext, sizeof infotext,
-                     "(xinerama screen: %i)", user_screen);
-            if (center) center_image(q);
-            update_image(q, FULL_REDRAW);
-          break;
-#else
-          case 'X':
-          {
-            int numlines = 0;
-            const char **lines;
-            run_command(q, ev->key.string, image_names[image_idx], &numlines, &lines);
-            if (lines && numlines)
-              qiv_display_text_window(q, "(Command output)", lines, "Push any key...");
-          }
-          break;
-#endif
+            break;
+          case ',':
+            if(fullscreen) 
+            {
+              disable_grab ^= 1;
+              snprintf(infotext, sizeof infotext, "(grab %s)", (disable_grab ? "off" : "on"));
+            }
+            else
+            {
+              do_grab ^= 1;
+              snprintf(infotext, sizeof infotext, "(grab %s)", (do_grab ? "on" : "off"));
+            }
+            if((!fullscreen && do_grab) || (fullscreen && !disable_grab))
+            {
+              gdk_keyboard_grab(q->win, FALSE, CurrentTime);
+              gdk_pointer_grab(q->win, FALSE,
+                  GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK |
+                  GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK, NULL, NULL, CurrentTime);
+            }
+            else
+            {
+              gdk_pointer_ungrab(CurrentTime);
+              gdk_keyboard_ungrab(CurrentTime);
+            }
+            update_image(q, MIN_REDRAW);
+            break;
 
             /* run qiv-command */
           case '^':    // special command with options
@@ -1063,7 +1065,9 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
           case '9':
           case 'A':
           case 'D':
+#ifndef HAVE_EXIF
           case 'E':
+#endif
           case 'F':
           case 'H':
           case 'J':
@@ -1102,5 +1106,6 @@ void qiv_handle_event(GdkEvent *ev, gpointer data)
   }
   if (exit_slideshow) {
     slide=0;
+    dpms_enable();
   }
 }
