@@ -37,6 +37,12 @@ Imlib_Image im_from_pixbuf_loader(char * image_name, int * has_alpha)
   int pb_w, pb_h;
   Imlib_Image * im=NULL;
   const gchar *gdk_orientation = NULL;
+#ifdef SUPPORT_LCMS
+  char *icc_profile;
+  cmsHPROFILE h_emb_profile;
+  cmsHTRANSFORM h_emb_transform;
+#endif
+
 
   pixbuf_ori = gdk_pixbuf_new_from_file(image_name, &error);
   if (error != NULL)
@@ -135,6 +141,46 @@ Imlib_Image im_from_pixbuf_loader(char * image_name, int * has_alpha)
         }
       }
     }
+
+#ifdef SUPPORT_LCMS
+    if((icc_profile=get_icc_profile(image_name)))
+    {
+      h_emb_profile = cmsOpenProfileFromMem(icc_profile+sizeof(cmsUInt32Number), *(cmsUInt32Number *)icc_profile);
+      if(h_display_profile==NULL)
+      {
+        h_display_profile = cmsCreate_sRGBProfile();
+      }
+
+      h_emb_transform = cmsCreateTransform(h_emb_profile,
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+          TYPE_BGRA_8,
+          h_display_profile,
+          TYPE_BGRA_8,
+#else
+          TYPE_ARGB_8,
+          h_display_profile,
+          TYPE_ARGB_8,
+#endif
+          INTENT_PERCEPTUAL, 0);
+      if(h_emb_transform)
+      {
+        cmsDoTransform(h_emb_transform, argbdata, argbdata, pb_w*pb_h);
+        cmsCloseProfile(h_emb_profile);
+        cmsDeleteTransform(h_emb_transform);
+      }
+      else
+      {
+        printf("qiv warning: %s contains corrupt color profile\n",image_name);
+      }
+      free(icc_profile);
+    }
+
+    /* do the color transform */
+    else if (cms_transform && h_cms_transform)
+    {
+      cmsDoTransform(h_cms_transform, argbdata, argbdata, pb_w*pb_h);
+    }
+#endif
     im = imlib_create_image_using_copied_data(pb_w, pb_h, (DATA32*)argbdata);
     free(argbdata);
     if(*has_alpha)
@@ -155,7 +201,7 @@ void qiv_load_image(qiv_image *q)
   struct stat statbuf;
   const char * image_name = image_names[ image_idx];
   Imlib_Image * im=NULL;
-  int has_alpha=0;
+  int has_alpha=0, rot;
 
   gettimeofday(&load_before, 0);
 
@@ -164,6 +210,7 @@ void qiv_load_image(qiv_image *q)
 
   stat(image_name, &statbuf);
   current_mtime = statbuf.st_mtime;
+  file_size = statbuf.st_size;
 
 #ifdef DEBUG
   g_print("loading %s\n",image_name);
@@ -189,24 +236,36 @@ void qiv_load_image(qiv_image *q)
     q->orig_h = imlib_image_get_height();
   }
 
-  if (rotation>0) {
-    imlib_image_orientate(rotation);
-    if (rotation!=2) {
+  if (rotation > 10) {
+    /* conditional rotation -- apply rotation only if image fits better */
+    int screen_is_wide = monitor[q->mon_id].width > monitor[q->mon_id].height;
+    int image_is_wide = q->orig_w > q->orig_h;
+    int does_not_fit = q->orig_w > monitor[q->mon_id].width || q->orig_h > monitor[q->mon_id].height;
+    if (screen_is_wide != image_is_wide && does_not_fit)
+      rot = rotation - 10; /* we want the rotation (will be 11 -> 1 or 13 -> 3) */
+    else
+      rot = 0; /* don't rotate */
+  } else
+    rot = rotation;
+
+  if (rot) {
+    imlib_image_orientate(rot);
+    if (rot != 2) {
       swap(&q->orig_w, &q->orig_h);
       swap(&q->win_w, &q->win_h);
     }
   }
 
-  check_size(q, TRUE);
 
-  if (rotation>0) {
+  if (rot && rot != 2)
     correct_image_position(q);
-  }
 
   if (first) {
     setup_win(q);
     first = 0;
   }
+  
+  check_size(q, TRUE);
 
   /* desktop-background -> exit */
   if (to_root || to_root_t || to_root_s) {
@@ -220,12 +279,6 @@ void qiv_load_image(qiv_image *q)
 
   gdk_window_set_background(q->win, im ? &image_bg : &error_bg);
 
-  if (do_grab || (fullscreen && !disable_grab) ) {
-    gdk_keyboard_grab(q->win, FALSE, CurrentTime);
-    gdk_pointer_grab(q->win, FALSE,
-      GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK |
-      GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK, NULL, NULL, CurrentTime);
-  }
   gettimeofday(&load_after, 0);
   load_elapsed = ((load_after.tv_sec +  load_after.tv_usec / 1.0e6) -
                  (load_before.tv_sec + load_before.tv_usec / 1.0e6));
@@ -276,9 +329,9 @@ static void setup_win(qiv_image *q)
 {
   GdkWindowAttr attr;
   GdkPixmap *cursor_pixmap;
-  static GdkPixbuf *icon=NULL;;
-  static GdkPixmap *icon_pixmap;
-  static GdkBitmap *icon_bitmap;
+  static GList  icon_list = {NULL, NULL, NULL};
+
+  destroy_image(q);
 
   if (!fullscreen) {
     attr.window_type=GDK_WINDOW_TOPLEVEL;
@@ -290,14 +343,6 @@ static void setup_win(qiv_image *q)
     attr.height = q->win_h;
     q->win = gdk_window_new(NULL, &attr, GDK_WA_X|GDK_WA_Y);
 
-    /* create the icon only once */
-    if(icon==NULL)
-    {
-      icon = gdk_pixbuf_new_from_inline (-1, qiv_icon, FALSE, NULL);
-      gdk_pixbuf_render_pixmap_and_mask(icon, &icon_pixmap, &icon_bitmap, 10);
-    }
-    gdk_window_set_icon(q->win, NULL, icon_pixmap, icon_bitmap);
-                                             
     if (center) {
       GdkGeometry geometry = {
         .min_width = q->win_w,
@@ -325,30 +370,46 @@ static void setup_win(qiv_image *q)
 
   } else { /* fullscreen */
 
-    attr.window_type=GDK_WINDOW_TEMP;
+    attr.window_type=GDK_WINDOW_TOPLEVEL;
     attr.wclass=GDK_INPUT_OUTPUT;
     attr.event_mask=GDK_ALL_EVENTS_MASK;
-    attr.x = attr.y = 0;
-    attr.width=screen_x;
-    attr.height=screen_y;
+//    attr.x = 0;
+//    attr.y = 0;
+    attr.x = monitor[q->mon_id].x;
+    attr.y = monitor[q->mon_id].y;
+//    printf("mon_id %d \n",q->mon_id);
+    attr.width=monitor[q->mon_id].width;
+    attr.height=monitor[q->mon_id].height;
+
+//    q->win = gdk_window_new(NULL, &attr, GDK_WA_X|GDK_WA_Y|GDK_WA_TYPE_HINT);
     q->win = gdk_window_new(NULL, &attr, GDK_WA_X|GDK_WA_Y);
     gdk_window_set_cursor(q->win, cursor);
     if (!(to_root || to_root_t || to_root_s))
+    {
+      gdk_window_fullscreen(q->win);
       gdk_window_show(q->win);
+    }
   }
 
+  /* this is all done only once */
+  if(icon_list.data==NULL)
+  {
+    icon_list.data = gdk_pixbuf_new_from_inline (-1, qiv_icon, FALSE, NULL);
+    cursor_pixmap = gdk_bitmap_create_from_data(q->win, blank_cursor, 1, 1);
+    invisible_cursor = gdk_cursor_new_from_pixmap(cursor_pixmap, cursor_pixmap,
+        &text_bg, &text_bg, 0, 0);
+    cursor = visible_cursor = gdk_cursor_new(CURSOR);
+  }
+
+  gdk_window_set_icon_list(q->win, &icon_list);
+  gdk_window_set_cursor(q->win, cursor);
+
   q->bg_gc = gdk_gc_new(q->win);
   q->text_gc = gdk_gc_new(q->win); /* black is default */
   q->status_gc = gdk_gc_new(q->win);
   gdk_gc_set_foreground(q->bg_gc, &image_bg);
   gdk_gc_set_foreground(q->status_gc, &text_bg);
 
-  cursor_pixmap = gdk_bitmap_create_from_data(q->win, blank_cursor, 1, 1);
-  invisible_cursor = gdk_cursor_new_from_pixmap(cursor_pixmap, cursor_pixmap,
-                                                &text_bg, &text_bg, 0, 0);
-  cursor = visible_cursor = gdk_cursor_new(CURSOR);
-  gdk_window_set_cursor(q->win, cursor);
-
   setup_imlib_for_drawable(GDK_DRAWABLE(q->win));
 }
 
@@ -403,10 +464,10 @@ void set_desktop_image(qiv_image *q)
       gdk_window_set_back_pixmap(root_win, p, FALSE);
     } else {
       GdkGC *rootGC;
-      buffer = xcalloc(1, screen_x * screen_y * gvis->depth / 8);
+      buffer = xcalloc(1, screen_x * screen_y * gdk_visual_get_depth(gvis) / 8);
       rootGC = gdk_gc_new(root_win);
       temp = gdk_pixmap_create_from_data(root_win, buffer, screen_x,
-                                         screen_y, gvis->depth, &image_bg, &image_bg);
+                                         screen_y, gdk_visual_get_depth(gvis), &image_bg, &image_bg);
      gdk_drawable_set_colormap(GDK_DRAWABLE(temp),
 			      gdk_drawable_get_colormap(GDK_DRAWABLE(root_win)));
       gdk_draw_drawable(temp, rootGC, p, 0, 0, root_x, root_y, root_w, root_h);
@@ -498,13 +559,9 @@ void zoom_out(qiv_image *q)
 
 void zoom_maxpect(qiv_image *q)
 {
-#ifdef GTD_XINERAMA
-  double zx = (double)preferred_screen->width / (double)q->orig_w;
-  double zy = (double)preferred_screen->height / (double)q->orig_h;
-#else
-  double zx = (double)screen_x / (double)q->orig_w;
-  double zy = (double)screen_y / (double)q->orig_h;
-#endif
+  double zx = (double)monitor[q->mon_id].width / (double)q->orig_w;
+  double zy = (double)monitor[q->mon_id].height / (double)q->orig_h;
+
   /* titlebar and frames ignored on purpose to use full height/width of screen */
   q->win_w = (gint)(q->orig_w * MIN(zx, zy));
   q->win_h = (gint)(q->orig_h * MIN(zx, zy));
@@ -558,12 +615,27 @@ void reload_image(qiv_image *q)
 
 void check_size(qiv_image *q, gint reset)
 {
-  if (maxpect || (scale_down && (q->orig_w>screen_x || q->orig_h>screen_y))) {
+  if (maxpect || (scale_down && (q->orig_w>monitor[q->mon_id].width || q->orig_h>monitor[q->mon_id].height))) {
     zoom_maxpect(q);
   } else if (reset || (scale_down && (q->win_w<q->orig_w || q->win_h<q->orig_h))) {
     reset_coords(q);
   }
-  if (center) center_image(q);
+  if (center){
+    center_image(q);
+  }
+  else if (fullscreen) { 
+    if (q->win_x >  monitor[q->mon_id].x)
+      q->win_x -= monitor[q->mon_id].x;
+    if (q->win_y >  monitor[q->mon_id].y)
+      q->win_y -= monitor[q->mon_id].y;
+  }
+  else {
+    if (q->win_x <  monitor[q->mon_id].x)
+      q->win_x += monitor[q->mon_id].x;
+    if (q->win_y <  monitor[q->mon_id].y)
+      q->win_y += monitor[q->mon_id].y;
+  }
+
 }
 
 void reset_coords(qiv_image *q)
@@ -585,9 +657,9 @@ void reset_coords(qiv_image *q)
 
 void update_image(qiv_image *q, int mode)
 {
-  GdkPixmap * m = NULL;
+  static GdkPixmap * m = NULL;
   Pixmap x_pixmap, x_mask;
-  double elapsed;
+  double elapsed=0;
   struct timeval before, after;
   int i;
 
@@ -624,11 +696,12 @@ void update_image(qiv_image *q, int mode)
     if (mode == MOVED) {
       if (transparency && used_masks_before) {
         /* there should be a faster way to update the mask, but how? */
-	if (q->p)
-	{
-	  imlib_free_pixmap_and_mask(GDK_PIXMAP_XID(q->p));
-	  g_object_unref(q->p);
-	}
+        if (q->p)
+        {
+          imlib_free_pixmap_and_mask(GDK_PIXMAP_XID(q->p));
+          g_object_unref(q->p);
+        }
+        if (m) g_object_unref(m);
 	imlib_render_pixmaps_for_whole_image_at_size(&x_pixmap, &x_mask, q->win_w, q->win_h);
 	q->p = gdk_pixmap_foreign_new(x_pixmap);
 	gdk_drawable_set_colormap(GDK_DRAWABLE(q->p),
@@ -646,23 +719,29 @@ void update_image(qiv_image *q, int mode)
     } // mode == MOVED
     else
     {
-      if (q->p) {
-	imlib_free_pixmap_and_mask(GDK_PIXMAP_XID(q->p));
-	g_object_unref(q->p);
+      if(mode != MIN_REDRAW)
+      {
+        if (q->p) {
+          imlib_free_pixmap_and_mask(GDK_PIXMAP_XID(q->p));
+          g_object_unref(q->p);
+        }
+        if (m) g_object_unref(m);
+
+        /* calculate elapsed time while we render image */
+        gettimeofday(&before, 0);
+        imlib_render_pixmaps_for_whole_image_at_size(&x_pixmap, &x_mask, q->win_w, q->win_h);
+        gettimeofday(&after, 0);
+        elapsed = ((after.tv_sec +  after.tv_usec / 1.0e6) -
+            (before.tv_sec + before.tv_usec / 1.0e6));
+
+        /*TODO: Hier gibt es XID collision, wenn am Bild eigentlich nix geändert wurde*/
+        q->p = gdk_pixmap_foreign_new_for_screen(screen, x_pixmap, q->win_w, q->win_h, 24);
+        gdk_drawable_set_colormap(GDK_DRAWABLE(q->p),
+            gdk_drawable_get_colormap(GDK_DRAWABLE(q->win)));
+//        m = x_mask == None ? NULL : gdk_pixmap_foreign_new(x_mask);
+        m = x_mask == None ? NULL : gdk_pixmap_foreign_new_for_screen(screen, x_mask, q->win_w, q->win_h, 1);
       }
 
-      /* calculate elapsed time while we render image */
-      gettimeofday(&before, 0);
-      imlib_render_pixmaps_for_whole_image_at_size(&x_pixmap, &x_mask, q->win_w, q->win_h);
-      gettimeofday(&after, 0);
-      elapsed = ((after.tv_sec +  after.tv_usec / 1.0e6) -
-                 (before.tv_sec + before.tv_usec / 1.0e6));
-
-      q->p = gdk_pixmap_foreign_new(x_pixmap);
-      gdk_drawable_set_colormap(GDK_DRAWABLE(q->p),
-				gdk_drawable_get_colormap(GDK_DRAWABLE(q->win)));
-      m = x_mask == None ? NULL : gdk_pixmap_foreign_new(x_mask);
-
 #ifdef DEBUG
       if (m)  g_print("*** image has transparency\n");
 #endif
@@ -715,13 +794,8 @@ void update_image(qiv_image *q, int mode)
   } // if (!fullscreen)
   else
   {
-#ifdef GTD_XINERAMA
-# define statusbar_x (statusbar_screen->x_org + statusbar_screen->width)
-# define statusbar_y (statusbar_screen->y_org + statusbar_screen->height)
-#else
-# define statusbar_x screen_x
-# define statusbar_y screen_y
-#endif
+# define statusbar_x monitor[q->mon_id].width
+# define statusbar_y monitor[q->mon_id].height
     if (mode == FULL_REDRAW) {
       gdk_window_clear(q->win);
     } else {
@@ -785,6 +859,8 @@ void update_image(qiv_image *q, int mode)
     q->text_ow = q->text_w;
     q->text_oh = q->text_h;
     q->statusbar_was_on = statusbar_fullscreen;
+    gdk_window_move_resize(q->win, monitor[q->mon_id].x, monitor[q->mon_id].y,
+        monitor[q->mon_id].width, monitor[q->mon_id].height);
   }
   gdk_flush();
 }
@@ -803,9 +879,16 @@ void destroy_image(qiv_image *q)
     imlib_free_pixmap_and_mask(GDK_PIXMAP_XID(q->p));
     g_object_unref(q->p);
   }
+  if (q->win) g_object_unref(q->win);
   if (q->bg_gc) g_object_unref(q->bg_gc);
   if (q->text_gc) g_object_unref(q->text_gc);
   if (q->status_gc) g_object_unref(q->status_gc);
+
+  q->p=NULL;
+  q->win=NULL;
+  q->bg_gc=NULL;
+  q->text_gc=NULL;
+  q->status_gc=NULL;
 }
 
 void setup_magnify(qiv_image *q, qiv_mgl *m)
@@ -814,6 +897,7 @@ void setup_magnify(qiv_image *q, qiv_mgl *m)
    GdkGeometry mgl_hints;
 
    m->win_w=300; m->win_h=200;
+   m->zoom=2.0;
 
 //   gdk_flush();
    gdk_window_get_root_origin(q->win, &m->frame_x, &m->frame_y);
@@ -821,7 +905,7 @@ void setup_magnify(qiv_image *q, qiv_mgl *m)
 
    mgl_attr.window_type=GDK_WINDOW_TOPLEVEL; // Set up attributes for GDK to create a Window
    mgl_attr.wclass=GDK_INPUT_OUTPUT;
-   mgl_attr.event_mask=GDK_STRUCTURE_MASK;
+   mgl_attr.event_mask=GDK_STRUCTURE_MASK | GDK_EXPOSURE_MASK;
    mgl_attr.width=m->win_w;
    mgl_attr.height=m->win_h;
    mgl_attr.override_redirect=TRUE;
@@ -865,36 +949,30 @@ void update_magnify(qiv_image *q, qiv_mgl *m, int mode, gint xcur, gint ycur)
   }
 ************/
   if (mode == REDRAW ) {
-    xx=xcur * q->orig_w / q->win_w;    /* xx, yy are the coords of cursor   */
-    if (xx <= m->win_w/2)               /* xcur, ycur scaled to the original */
-      xx=0;                             /* image; they are changed so that   */
-    else                                /* the magnified part is always      */
-      if (xx >= q->orig_w - m->win_w/2) /* inside the image.                 */
-        xx=q->orig_w - m->win_w;
-      else
-        xx=xx - m->win_w/2;
 
-    yy=ycur * q->orig_h / q->win_h;
-    if (yy <= m->win_h/2)
-      yy=0;
-    else
-      if (yy >= q->orig_h - m->win_h/2)
-        yy=q->orig_h - m->win_h;
-      else
-        yy=yy - m->win_h/2;
+    /* scale position to original size */
+    xx=xcur * ((double)q->orig_w/(double)q->win_w);
+    yy=ycur * ((double)q->orig_h/(double)q->win_h);
+
+    /* center mouse cursor position */
+    xx -= 150 / m->zoom;
+    yy -= 100 / m->zoom;
+
+    /* keep magnify part allways inside image */
+    if(xx + m->win_w / m->zoom > q->orig_w) {
+      xx=q->orig_w - m->win_w / m->zoom;
+    }
+    if(yy + m->win_h / m->zoom > q->orig_h) {
+      yy=q->orig_h - m->win_h / m->zoom;
+    }
 
-//    printf("MGL: xcur: %d, ycur: %d, xx: %d, yy: %d\n", xcur, ycur, xx, yy);
 
     setup_imlib_for_drawable(m->win);
-    imlib_render_image_part_on_drawable_at_size(xx, yy, m->win_w, m->win_h,
+    imlib_render_image_part_on_drawable_at_size(xx, yy, m->win_w / m->zoom , m->win_h / m->zoom,
 						0, 0, m->win_w, m->win_h);
     setup_imlib_for_drawable(q->win);
     gdk_window_show(m->win);
 
-    // xcur= m->frame_x + xcur +
-    // (xcur < m->win_w/2? 50 : - 50 - m->win_w);
-    // gdk_window_get_root_origin(q->win,              // todo  [lc]
-    //        &magnify_img.frame_x, &magnify_img.frame_y);  // call not necessary
     xx= m->frame_x + xcur - 50 - m->win_w;
     yy= m->frame_y + ycur - 50 - m->win_h;
     if (xx < 0) {
@@ -910,64 +988,22 @@ void update_magnify(qiv_image *q, qiv_mgl *m, int mode, gint xcur, gint ycur)
         yy=0;
     }
 //    printf("MGL: m->frame_x: %d, m->frame_y: %d, xx: %d, yy: %d\n", m->frame_x, m->frame_y, xx, yy);
-
-//    gdk_window_move_resize(m->win, xx, yy,    m->win_w, m->win_h);
     gdk_window_move(m->win, xx, yy);
   }
-
-// gdk_window_show(m->win);
   gdk_flush();
 }
 
-#ifdef GTD_XINERAMA
 void center_image(qiv_image *q)
 {
-//  g_print("before: q->win_x = %d, q->win_y = %d, q->win_w = %d\n", q->win_x, q->win_y, q->win_w);
-  XineramaScreenInfo * pscr = preferred_screen;
 
-//  g_print("screen_x = %d, pscr->x_org = %d, pscr->width = %d\n", screen_x, pscr->x_org, pscr->width);
-
-  /* Figure out x position */
-  if (q->win_w <= pscr->width) {
-    /* If image fits on screen try to center image
-     * within preferred (sub)screen */
-    q->win_x = (pscr->width - q->win_w) / 2 + pscr->x_org;
-    /* Ensure image actually lies within screen boundaries */
-    if (q->win_x < 0) {
-      q->win_x = 0;
-    }
-    else if (q->win_x + q->win_w > screen_x) {
-      q->win_x = screen_x - q->win_w;
-    }
+  q->win_x = (monitor[q->mon_id].width  - q->win_w) / 2;
+  q->win_y = (monitor[q->mon_id].height - q->win_h) / 2;
+  if(!fullscreen)
+  {
+    q->win_x += monitor[q->mon_id].x;
+    q->win_y += monitor[q->mon_id].y;
   }
-  else {
-    /* If image wider than screen, just center it over all screens */
-    q->win_x = (screen_x - q->win_w) / 2;
-//    g_print("q->win_x = %d, screen_x = %d, q->win_w = %d, pscr->width = %d\n", q->win_x, screen_x, q->win_w, pscr->width);
-  }
-
-  /* Same thing for y position */
-  if (q->win_h <= screen_y) {
-    q->win_y = (pscr->height - q->win_h) / 2 + pscr->y_org;
-    if (q->win_y < 0) {
-      q->win_y = 0;
-    }
-    else if (q->win_y + q->win_h > screen_y) {
-      q->win_y = screen_y - q->win_h;
-    }
-  }
-  else {
-    q->win_y = (screen_y - q->win_h) / 2;
-  }
-//  g_print("after:  q->win_x = %d, q->win_y = %d, q->win_w = %d\n", q->win_x, q->win_y, q->win_w);
 }
-#else
-void center_image(qiv_image *q)
-{
-  q->win_x = (screen_x - q->win_w) / 2;
-  q->win_y = (screen_y - q->win_h) / 2;
-}
-#endif
 
 void correct_image_position(qiv_image *q)
 {
