@@ -24,7 +24,6 @@
 qiv_image main_img;
 qiv_mgl   magnify_img; /* [lc] */
 
-static void filter_images(int *, char **);
 static int check_extension(const char *);
 static void qiv_signal_usr1();
 static void qiv_signal_usr2();
@@ -32,12 +31,13 @@ static gboolean qiv_handle_timer(gpointer);
 static void qiv_timer_restart(gpointer);
 
 #ifdef HAVE_MAGIC
-static int check_magic(const char *name);
+static int check_magic(magic_t cookie, const char *name);
 #endif
 
 int main(int argc, char **argv)
 {
   struct timeval tv;
+  int i;
 
   // [as] workaround for problem with X composite extension
   // is this still needed with imlib2 ??
@@ -66,8 +66,51 @@ int main(int argc, char **argv)
   /* Set up our options, image list, etc */
   strncpy(select_dir, SELECT_DIR, sizeof select_dir);
   reset_mod(&main_img);
+
   options_read(argc, argv, &main_img);
 
+#ifdef SUPPORT_LCMS
+  /* read profiles if provided */
+  if (cms_transform) 
+  {
+    if (source_profile) {
+      h_source_profile = cmsOpenProfileFromFile(source_profile, "r");
+    } else {
+      h_source_profile = cmsCreate_sRGBProfile();
+    }
+
+    if (h_source_profile == NULL)
+    {
+      g_print("qiv: cannot create source color profile.\n");
+      usage(argv[0],1);
+    }
+      
+    if (display_profile) {
+      h_display_profile = cmsOpenProfileFromFile(display_profile, "r");
+    } else {
+      h_display_profile = cmsCreate_sRGBProfile();
+    }
+    if (h_display_profile == NULL)
+    {
+      g_print("qiv: cannot create display color profile.\n");
+      usage(argv[0],1);
+    }
+
+  }
+  /* TYPE_BGRA_8 or TYPE_ARGB_8 depending on endianess */
+  h_cms_transform = cmsCreateTransform(h_source_profile,
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+				       TYPE_BGRA_8,
+				       h_display_profile,
+				       TYPE_BGRA_8,
+#else
+				       TYPE_ARGB_8,
+				       h_display_profile,
+				       TYPE_ARGB_8,
+#endif
+				       INTENT_PERCEPTUAL, 0);
+#endif
+
   /* Load things from GDK/Imlib */
 
   qiv_main_loop = g_main_new(TRUE);
@@ -75,6 +118,19 @@ int main(int argc, char **argv)
   screen_x = gdk_screen_width();
   screen_y = gdk_screen_height();
 
+  screen  = gdk_screen_get_default();
+  num_monitors = gdk_screen_get_n_monitors(screen);
+  monitor = malloc( num_monitors * sizeof(GdkRectangle));
+  for(i=0; i< num_monitors ; i++)
+  {
+      gdk_screen_get_monitor_geometry(screen, i, &monitor[i]);
+  }
+  
+  if(user_screen < num_monitors)
+  {
+    main_img.mon_id = user_screen;
+  }
+
  /* statusbar with pango */
   layout = pango_layout_new(gdk_pango_context_get()); 
   fontdesc = pango_font_description_from_string (STATUSBAR_FONT);
@@ -87,33 +143,8 @@ int main(int argc, char **argv)
   metrics = pango_context_get_metrics (gdk_pango_context_get(), fontdesc, NULL);
   pango_layout_set_font_description (layout, fontdesc); 
 
-/*
-  // [as] thinks that this is not portable enough
-  // [lc]
-  // I use a virtual screen of 1600x1200, and the resolution is 1024x768,
-  // so I changed how screen_[x,y] is obtained; it seems that gtk 1.2
-  // cannot give the geometry of viewport, so I borrowed from the source of
-  // xvidtune the code for calling XF86VidModeGetModeLine, this requires
-  // the linking option -lXxf86vm.
-  XF86VidModeGetModeLine(GDK_DISPLAY(), DefaultScreen(GDK_DISPLAY()),
-                         &dot_clock, &modeline);
-  //printf("> hdisplay %d vdisplay %d\n", modeline.hdisplay, modeline.vdisplay);
-  screen_x=MIN(screen_x, modeline.hdisplay);
-  screen_y=MIN(screen_y, modeline.vdisplay);
-*/
-
-#ifdef GTD_XINERAMA
-  get_preferred_xinerama_screens();
-//  screen_x=MIN(screen_x, preferred_screen->width);
-//  screen_y=MIN(screen_y, preferred_screen->height);
-#endif
-
-
   max_rand_num = images;
 
-  if (filter) /* Filter graphic images */
-    filter_images(&images,image_names);
-
   if (!images) { /* No images to display */
     g_print("qiv: cannot load any images.\n");
     usage(argv[0],1);
@@ -148,6 +179,14 @@ int main(int argc, char **argv)
   signal(SIGUSR1, qiv_signal_usr1);
   signal(SIGUSR2, qiv_signal_usr2);
 
+  /* check for DPMS capability and disable it if slideshow
+     was started from command options */
+  dpms_check();
+  if(slide){
+    dpms_disable();
+  }
+
+
   /* Load & display the first image */
 
   qiv_load_image(&main_img);
@@ -165,6 +204,7 @@ void qiv_exit(int code)
 {
   if (cmap) gdk_colormap_unref(cmap);
   destroy_image(&main_img);
+  dpms_enable();
 
   pango_font_description_free (fontdesc);
   g_object_unref (layout);
@@ -205,6 +245,8 @@ static gboolean qiv_handle_timer(gpointer data)
 {
   if (*(int *)data || slide) {
     next_image(0);
+    /* disable screensaver during slideshow */
+    XResetScreenSaver(GDK_DISPLAY());
     qiv_load_image(&main_img);
   }
   return FALSE;
@@ -224,14 +266,20 @@ static void qiv_timer_restart(gpointer dummy)
 
 /* Filter images by extension */
 
-static void filter_images(int *images, char **image_names)
+void filter_images(int *images, char **image_names)
 {
   int i = 0;
+#ifdef HAVE_MAGIC
+  magic_t cookie;
+
+  cookie = magic_open(MAGIC_NONE);
+  magic_load(cookie,NULL);
+#endif
 
   while(i < *images) {
     if (check_extension(image_names[i])
 #ifdef HAVE_MAGIC
-            || check_magic(image_names[i])
+            || check_magic(cookie, image_names[i])
 #endif
        ) {
       i++;
@@ -246,6 +294,9 @@ static void filter_images(int *images, char **image_names)
       --(*images);
     }
   }
+#ifdef HAVE_MAGIC
+  magic_close(cookie);
+#endif
   if (image_idx < 0)
     image_idx = 0;
 }
@@ -264,15 +315,12 @@ static int check_extension(const char *name)
 }
 
 #ifdef HAVE_MAGIC
-static int check_magic(const char *name)
+static int check_magic(magic_t cookie, const char *name)
 {
-  magic_t cookie;
   const char *description=NULL;
   int i;
   int ret=0;
 
-  cookie = magic_open(MAGIC_NONE);
-  magic_load(cookie,NULL);
   description = magic_file(cookie, name);
   if(description)
   {
@@ -283,7 +331,6 @@ static int check_magic(const char *name)
         break;
       }
   }
-  magic_close(cookie);
   return ret;
 }
 #endif
