@@ -5,6 +5,10 @@ int		first = 1; /* TRUE if this is first image or after fullscreen or iconifying
 char		infotext[BUF_LEN];
 GMainLoop	*qiv_main_loop;
 gint		screen_x, screen_y; /* Size of the screen in pixels */
+gint            num_monitors;
+GdkScreen       *screen;
+GdkRectangle    *monitor;
+
 GdkColormap	*cmap; /* global colormap */
 char		*image_bg_spec = IMAGE_BG;
 GdkColor	image_bg; /* default background */
@@ -18,6 +22,9 @@ time_t          current_mtime; /* modification time of file currently loaded */
 qiv_deletedfile *deleted_files;
 int		delete_idx;
 char    select_dir[FILENAME_LEN];
+char    *comment=NULL; 
+gint    jpeg_prog=0;
+off_t   file_size;
 
 /* stuff for rendering statusbar, infotext, etc ... */
 PangoLayout     *layout;
@@ -43,27 +50,32 @@ int	statusbar_window = 0; /* FALSE if statusbar in window is turned off (default
 int	slide; /* 1=slide show running */
 int	scale_down; /* resize down if image x/y > screen */
 int	recursive; /* descend recursively */
+int	followlinks; /* follow symlinks to dirs */
 int	to_root; /* display on root (centered) */
 int	to_root_t; /* display on root (tiled) */
 int	to_root_s; /* display on root (stretched) */
 int	transparency; /* transparency on/off */
 int	do_grab; /* grab keboard/pointer (default off) */
-int disable_grab; /* disable keyboard/mouse grabbing in fullscreen mode */
+int	disable_grab; /* disable keyboard/mouse grabbing in fullscreen mode */
 int	max_rand_num; /* the largest random number range we will ask for */
 int	fixed_window_size = 0; /* window width fixed size/off */
 int	fixed_zoom_factor = 0; /* window fixed zoom factor (percentage)/off */
 int zoom_factor = 0; /* zoom factor/off */
 int watch_file = 0; /* watch current files Timestamp, reload if changed */
 int magnify = 0; /* [lc] */
-int user_screen = -1; /* preferred (by user) Xinerama screen */
+int user_screen = 0; /* preferred (by user) monitor */
 int browse = 0; /* scan directory of file for browsing */
 int autorotate = 0; /* autorotate JPEGs according to EXIF tag */
 int rotation = 0; /* rotation x degrees clockwise, 1=90degrees 2=180degrees 3=270degrees */
+int vikeys = 0; /* option to give us some vi-like keys (for movement) */
 
-#ifdef GTD_XINERAMA
-int number_xinerama_screens = 0;
-XineramaScreenInfo preferred_screen[1];
-XineramaScreenInfo statusbar_screen[1];
+#ifdef SUPPORT_LCMS
+const char* source_profile = NULL;
+const char* display_profile = NULL;
+cmsHPROFILE h_source_profile;
+cmsHPROFILE h_display_profile;
+cmsHTRANSFORM h_cms_transform;
+int cms_transform = 0;
 #endif
 
 
@@ -92,7 +104,7 @@ const char *helpstrs[] =
     "f                    fullscreen mode on/off",
     "m                    scale to screen size on/off",
     "t                    scale down on/off",
-    "X                    cycle through xinerama screens",
+    "X                    cycle through monitors",
     "s                    slide show on/off",
     "p                    transparency on/off",
     "r                    random order on/off",
@@ -111,12 +123,16 @@ const char *helpstrs[] =
     "jfx<return>          jump forward x images",
     "jbx<return>          jump backward x images",
     "enter/return         reset zoom, rotation and color settings",
+#ifdef HAVE_EXIF
+    "E                    display Exif information",
+#endif
     "i                    statusbar on/off",
     "I                    iconify window",
     "w                    watch file on/off",
     "x                    center image on background",
     "y                    tile image on background",
     "z                    stretch image on background",
+    ",                    grab on/off",
     "<                    turn on/off magnifying window",
     "arrow keys                 move image (in fullscreen mode)",
     "arrow keys+Shift           move image faster (in fullscreen mode)",
