@@ -23,7 +23,9 @@
 extern char *optarg;
 extern int optind, opterr, optopt;
 
-static char *short_options = "ab:c:Cd:efg:hilmno:pq:rstuvw:xyzA:BDF:GIMNPRSTW:X:";
+#define LONGOPT_VIKEYS 128
+
+static char *short_options = "ab:c:Cd:efg:hilLmno:pq:rstuvw:xyzA:BDF:GIMNPRSTW:X:Y:Z:";
 static struct option long_options[] =
 {
     {"do_grab",          0, NULL, 'a'},
@@ -42,11 +44,12 @@ static struct option long_options[] =
     {"no_filter",        0, NULL, 'n'},
     {"bg_color",         1, NULL, 'o'},
     {"transparency",     0, NULL, 'p'},
-    {"rotation",         1, NULL, 'q'},
+    {"rotate",           1, NULL, 'q'},
     {"random",           0, NULL, 'r'},
     {"slide",            0, NULL, 's'},
     {"scale_down",       0, NULL, 't'},
     {"recursivedir",     0, NULL, 'u'},
+    {"followlinks",      0, NULL, 'L'},
     {"version",          0, NULL, 'v'},
     {"fixed_width",      1, NULL, 'w'},
     {"root",             0, NULL, 'x'},
@@ -67,6 +70,11 @@ static struct option long_options[] =
     {"watch",            0, NULL, 'T'},
     {"fixed_zoom",       1, NULL, 'W'},
     {"xineramascreen",   1, NULL, 'X'},
+#ifdef SUPPORT_LCMS
+    {"source_profile",   1, NULL, 'Y'},
+    {"display_profile",  1, NULL, 'Z'},
+#endif
+    {"vikeys",           0, NULL, LONGOPT_VIKEYS},
     {0,                  0, NULL, 0}
 };
 
@@ -248,7 +256,7 @@ void options_read(int argc, char **argv, qiv_image *q)
             case 'p': transparency=1;
                 break;
             case 'q': rotation=checked_atoi(optarg);
-                if ((rotation<0) || (rotation>3))
+                if (rotation < 0 || (rotation > 3 && rotation != 11 && rotation != 13))
                     usage(argv[0],1);
                 break;
             case 'r': random_order=1;
@@ -259,6 +267,8 @@ void options_read(int argc, char **argv, qiv_image *q)
                 break;
             case 'u': recursive = 1;
                 break;
+            case 'L': followlinks=1;
+                break;
             case 'v': g_print("qiv (Quick Image Viewer) v%s\n", VERSION);
                 gdk_exit(0);
                 break;
@@ -301,11 +311,19 @@ void options_read(int argc, char **argv, qiv_image *q)
                 break;
             case 'W': fixed_zoom_factor = (checked_atoi(optarg) - 100) / 10;
                 break;
-#ifdef GTD_XINERAMA
             case 'X': user_screen = checked_atoi(optarg);
 //               g_print("set xinerama screen: %i\n", user_screen);
                 break;
+#ifdef SUPPORT_LCMS
+            case 'Y': source_profile = optarg;
+	        cms_transform = 1;
+	        break;
+            case 'Z': display_profile = optarg;
+	        cms_transform = 1;
+	        break;
 #endif
+            case LONGOPT_VIKEYS: vikeys=1;
+              break;
             case 0:
             case '?': usage(argv[0], 1);
                       gdk_exit(0);
@@ -352,17 +370,28 @@ void options_read(int argc, char **argv, qiv_image *q)
         }
     }
 
-    if (browse) {
+    if (browse && images) {
         images = 0; /* avoid displaying same filename twice */
-        char *tmp = (char *)xmalloc(strlen(image_names[0])+1);
-        strcpy(tmp,image_names[0]);
-        rreaddir(dirname(image_names[0]),0);
-        if (need_sort) {
-            qsort(image_names, images, sizeof *image_names, my_strcmp);
+        char *tmp = (char *)xmalloc(strlen(image_names[0])+3);
+        if(strchr(image_names[0], '/')==NULL) {
+            *tmp     ='.';
+            *(tmp+1) ='/';
+            strcpy(tmp+2,image_names[0]);
         }
+        else {
+            strcpy(tmp,image_names[0]);
+        }
+        rreaddir(dirname(image_names[0]),0);
+        filter_images(&images,image_names);
+        if(need_sort)
+            qsort(image_names, images, sizeof *image_names, my_strcmp);
         image_idx = find_image(images,image_names,tmp);
         free(tmp);
-    } else if (need_sort) {
-        qsort(image_names, images, sizeof *image_names, my_strcmp);
     }
+    else {
+        filter_images(&images,image_names);
+        if(need_sort)
+            qsort(image_names, images, sizeof *image_names, my_strcmp);
+    }
+
 }
