@@ -8,9 +8,9 @@
 #include <Imlib2.h>
 #include <unistd.h>
 #include <stdlib.h>
-#ifdef GTD_XINERAMA
-# include <X11/Xlib.h>
-# include <X11/extensions/Xinerama.h>
+#ifdef SUPPORT_LCMS
+#include <lcms2.h>
+#include <jpeglib.h>
 #endif
 
 /*
@@ -25,8 +25,8 @@
 #include <X11/extensions/xf86vmode.h> // for XF86VidModeGetModeLine
 */
 
-#define VERSION "2.2.4"
-#define VERSION_FULL "QIV - Quick Image Viewer v2.2.4 - http://qiv.spiegl.de/"
+#define VERSION "2.3.1"
+#define VERSION_FULL "QIV - Quick Image Viewer v2.3.1 - http://qiv.spiegl.de/"
 #define TRASH_DIR ".qiv-trash"
 #define SELECT_DIR ".qiv-select"
 #define SLIDE_DELAY 3000 /* milliseconds */
@@ -56,7 +56,7 @@ typedef struct _qiv_image {
   GdkPixmap *p; /* Pixmap of the image to display */
   GdkWindow *win; /* Main window for windowed and fullscreen mode */
   int error; /* 1 if Imlib couldn't load image */
-  gint win_x, win_y, win_w, win_h; /* window co-ordinates */
+  gint win_x, win_y, win_w, win_h, mon_id; /* window co-ordinates */
   gint orig_w, orig_h; /* Size of original image in pixels */
   GdkGC *bg_gc;     /* image window background */
   GdkGC *text_gc;   /* statusbar text color */
@@ -82,6 +82,7 @@ typedef struct _qiv_mgl {
   GdkWindow *win; /* window for magnify */
   gint win_x, win_y, win_w, win_h; /* window coordinates */
   gint frame_x, frame_y; /* UpLeft Corner of frame (titlebar included) of */
+  double zoom;
                          /* main qiv_image */
 //  GdkGC *bg_gc;     /* image window background */
 //  int exposed;        /* window became visible */
@@ -96,6 +97,9 @@ extern int              first;
 extern char             infotext[BUF_LEN];
 extern GMainLoop        *qiv_main_loop;
 extern gint             screen_x, screen_y;
+extern gint             num_monitors;
+extern GdkScreen        *screen;
+extern GdkRectangle     *monitor;
 extern GdkFont          *text_font;
 extern GdkColormap      *cmap;
 extern char             *image_bg_spec;
@@ -113,6 +117,9 @@ extern char             select_dir[FILENAME_LEN];
 extern PangoLayout     *layout;
 extern PangoFontDescription *fontdesc;
 extern PangoFontMetrics *metrics;
+extern char             *comment;
+extern gint             jpeg_prog;
+extern off_t            file_size;
 
 extern int     filter;
 extern gint    center;
@@ -131,6 +138,7 @@ extern int     statusbar_window;
 extern int     slide;
 extern int     scale_down;
 extern int     recursive;
+extern int     followlinks;
 extern int     to_root;
 extern int     to_root_t;
 extern int     to_root_s;
@@ -147,21 +155,27 @@ extern int     magnify;     // [lc]
 extern qiv_mgl magnify_img; // [lc]
 extern int     autorotate;
 extern int     rotation;
+extern int     vikeys;
 
 extern const char   *helpstrs[], **helpkeys, *image_extensions[];
 
-#ifdef GTD_XINERAMA
-extern XineramaScreenInfo preferred_screen[1];
-extern XineramaScreenInfo statusbar_screen[1];
 extern int user_screen;
-extern int number_xinerama_screens;
-extern void get_preferred_xinerama_screens(void);
+
+#ifdef SUPPORT_LCMS
+extern const char* source_profile;
+extern const char* display_profile;
+extern cmsHPROFILE h_source_profile;
+extern cmsHPROFILE h_display_profile;
+extern cmsHTRANSFORM h_cms_transform;
+extern int cms_transform;
+extern char *get_icc_profile(char *filename);
 #endif
 
 /* main.c */
 
 extern void qiv_exit(int);
 extern void qiv_load_image();
+extern void filter_images(int *images, char **image_names);
 
 /* image.c */
 
@@ -170,6 +184,7 @@ extern void qiv_load_image();
 #define MOVED  1
 #define ZOOMED 2
 #define FULL_REDRAW 3
+#define MIN_REDRAW 4
 
 extern void qiv_load_image(qiv_image *);
 extern void set_desktop_image(qiv_image *);
@@ -219,3 +234,9 @@ extern gboolean qiv_watch_file (gpointer);
 extern int rreaddir(const char *, int);
 extern int rreadfile(const char *);
 extern int find_image(int images, char **image_names, char *name);
+#ifdef HAVE_EXIF
+extern char **get_exif_values(char *filename);
+#endif
+void dpms_check();
+void dpms_enable();
+void dpms_disable();
