@@ -17,6 +17,11 @@
 #include <dirent.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <tiffio.h>
+#include <X11/extensions/dpms.h>
+#ifdef HAVE_EXIF
+#include <libexif/exif-loader.h>
+#endif
 #include "qiv.h"
 #include "xmalloc.h"
 
@@ -346,7 +351,7 @@ void run_command(qiv_image *q, char *n, char *filename, int *numlines, const cha
   if (before.st_size == after.st_size &&
       before.st_ctime == after.st_ctime &&
       before.st_mtime == after.st_mtime)
-    update_image(q, FULL_REDRAW);
+    update_image(q, MIN_REDRAW);
   else
     qiv_load_image(q);
 }
@@ -407,6 +412,7 @@ void finish(int sig)
 {
   gdk_pointer_ungrab(CurrentTime);
   gdk_keyboard_ungrab(CurrentTime);
+  dpms_enable();
   exit(0);
 }
 
@@ -498,13 +504,16 @@ void show_help(char *name, int exit_status)
           "    --transparency, -p     Enable transparency for transparent images\n"
           "    --watch, -T            Reload the image if it has changed on disk\n"
           "    --recursivedir, -u     Recursively include all files\n"
+          "    --followlinks, -L      Follow symlinks to directories (requires --recursivedir)\n"
           "    --select_dir, -A x     Store the selected files in dir x (default is .qiv-select)\n"
 #if GDK_PIXBUF_MINOR >= 12
           "    --autorotate, -l       Autorotate JPEGs according to EXIF rotation tag\n"
 #endif
-          "    --rotate, -q x         Rotate 90(x=1),180(x=2),270(x=3) degrees clockwise\n"
-#ifdef GTD_XINERAMA
-          "    --xineramascreen, -X x Use screen x as preferred Xinerama screen\n"
+          "    --rotate, -q x         Rotate 90(x=1),180(x=2),270(x=3) degrees clockwise (11 & 13 for conditional)\n"
+          "    --xineramascreen, -X x Use monitor x as preferred screen\n"
+#ifdef SUPPORT_LCMS
+          "    --source_profile, -Y x Use color profile file x as source profile for all images\n"
+          "    --display_profile,-Z x Use color profile file x as display profile for all images\n"
 #endif
           "    --version, -v          Print version information and exit\n"
           "\n"
@@ -514,7 +523,8 @@ void show_help(char *name, int exit_status)
           "    --random, -r           Random order\n"
           "    --shuffle, -S          Shuffled order\n"
           "    --delay, -d x          Wait x seconds between images [default=%d]\n"
-          "\n"
+          "    --vikeys               Enable movement with h/j/k/l, vi-style\n"
+          "                           (HJKL will do what hjkl previously did)\n"          "\n"
           "Keys:\n", SLIDE_DELAY/1000);
 
     /* skip header and blank line */
@@ -592,7 +602,7 @@ int rreaddir(const char *dirname, int recursive)
         strcmp(entry->d_name,TRASH_DIR) == 0)
       continue;
     snprintf(name, sizeof name, "%s/%s", cdirname, entry->d_name);
-    if (lstat(name, &sb) >= 0) {
+    if ((followlinks ? stat(name, &sb) : lstat(name, &sb)) >= 0) {
       if (S_ISDIR(sb.st_mode)) {
         if (!recursive)
           continue;
@@ -706,8 +716,11 @@ gboolean qiv_watch_file (gpointer data)
   stat(image_names[image_idx], &statbuf);
 
   if(current_mtime!=statbuf.st_mtime && statbuf.st_size){
-	  reload_image(q);
-          update_image(q, REDRAW);
+          if(time(NULL)-statbuf.st_mtime > 0)
+          {
+              reload_image(q);
+              update_image(q, REDRAW);
+          }
   }
   usleep(200);  /* avoid eating 100% cpu */
 
@@ -724,97 +737,308 @@ int find_image(int images, char **image_names, char *name)
   return 0;
 }
 
-#ifdef GTD_XINERAMA
-/**
- * Find screen which maximizes f(screen)
- */
-static XineramaScreenInfo *
-xinerama_maximize_screen_function (XineramaScreenInfo * screens, int nscreens,
-                         long (*f)(XineramaScreenInfo *))
+#ifdef SUPPORT_LCMS
+char *get_icc_profile(char *filename)
 {
-  XineramaScreenInfo * screen;
-  long value;
-  long maxvalue = 0;
-  XineramaScreenInfo * maximal_screen = screens;
+  FILE *infile;
+  jpeg_saved_marker_ptr marker;
+  struct jpeg_decompress_struct cinfo;
+  struct jpeg_error_mgr jerr;
+  int j,i=0;
+  cmsUInt32Number length=0;
+  int jpg_ok;
+  unsigned char pic_tst[4];
 
-  for (screen = screens; nscreens--; screen++) {
-    //g_print("screen: %i\n", screen->screen_number);
-    value = f(screen);
-    if (value > maxvalue) {
-      maxvalue = value;
-      maximal_screen = screen;
+  char *icc_ptr=NULL;
+  unsigned short *tag_length=NULL;
+  unsigned char **tag_ptr=NULL;
+
+  /* Jpeg ICC header:
+   * Marker 0xffe2
+   * "ICC_PROFILE\0"
+   * Marker ID
+   * No of total markers
+   * data length
+   * data
+   */
+  const char icc_string[]="ICC_PROFILE";
+  int seq_max;
+
+  /* Tiff ICC header:
+   * Bytes
+   * 0-1 TIFFTAG_ICCPROFILE 34675(0x8773)
+   * 2-3 The field Type = 7
+   * 4-7 the size of the embedded ICC profile
+   * 8-11 Start of ICC profile in the tiff file
+   */
+
+  TIFF *tiff_image;
+
+  if ((infile = fopen(filename, "rb")) == NULL)
+  {
+    fprintf(stderr, "can't open %s\n", filename);
+    return NULL; 
+  }
+
+  if (fread(pic_tst, 1, 4, infile) != 4)
+  {
+    fclose(infile);
+    return NULL;
+  }
+
+  jpeg_prog=0;
+  if(comment)
+  {
+    free(comment);
+    comment=NULL;
+  }
+  
+  /* Is pic a jpg? */
+  if ( (pic_tst[0] == 0xff) && (pic_tst[1] == 0xd8) && (pic_tst[2] == 0xff) &&  ((pic_tst[3]& 0xf0) == 0xe0) )
+  {
+    rewind(infile);
+
+    cinfo.err = jpeg_std_error(&jerr);
+
+    jpeg_create_decompress(&cinfo);
+    jpeg_stdio_src(&cinfo, infile); 
+    jpeg_save_markers(&cinfo, 0xE2, 0xFFFF);
+    jpeg_save_markers(&cinfo, JPEG_COM, 0xFFFF);
+    jpg_ok = jpeg_read_header(&cinfo, 0);
+    fclose(infile);
+    jpeg_prog=cinfo.progressive_mode;
+
+    for (marker = cinfo.marker_list; marker != NULL; marker = marker->next) 
+    {
+      if(strncmp(icc_string, (const char *)marker->data, 11)==0)
+      {
+        if(i==0)
+        {
+          seq_max=marker->data[13];
+          tag_length = calloc(seq_max, sizeof(short));
+          tag_ptr    = calloc(seq_max, sizeof(char *));
+        }
+        // hmm, in theory both should be the same (tw)
+        tag_length[marker->data[12]-1] = marker->data_length - 14;
+//      tag_length[marker->data[12]-1] = (marker->data[16] << 8) +  marker->data[17];
+
+        (tag_ptr[marker->data[12]-1])  = marker->data;
+        (tag_ptr[marker->data[12]-1]) += 14;
+
+        i++;
+      }
+      /* copy jpeg comment here to be printed out when exif data is displayed. */
+      else if(marker->marker==JPEG_COM)
+      {
+        comment=calloc(1+marker->data_length,1);
+        if(comment==NULL) return NULL;
+        strncpy(comment, (char *) marker->data, marker->data_length);
+      }
     }
-  }
-  return maximal_screen;
-}
-
-static long
-xinerama_screen_number_pixels (XineramaScreenInfo * screen)
-{
-//  g_print("npixels: screen->width = %d, screen->height = %d\n", screen->width, screen->height);
-  return screen->width * screen->height;
-}
-
-/**
- * We will want to find the lower-rightmost screen (on which we
- * would like to display the statusbar in full-screen mode).
- *
- * In the general case (screens of differing sizes and arbitrarily placed)
- * the "lower-rightmost" screen is not particularly well defined.  We
- * take the definition as follows:
- *
- * Let (l_i, r_i) be the absolute pixel coordinates of the lower right corner
- * corner of screen i.  The lower-rightmost screen is the one for which
- * l_i + r_i is a maximum.
- */
-static long
-xinerama_screen_lower_rightness (XineramaScreenInfo * screen)
-{
-  return screen->x_org + screen->y_org + screen->width + screen->height;
-}
-
-void
-get_preferred_xinerama_screens(void)
-{
-  Display * dpy;
-  XineramaScreenInfo *screens = 0;
-
-  dpy = XOpenDisplay(gdk_get_display());
-  if (dpy && XineramaIsActive(dpy))
-    screens = XineramaQueryScreens(dpy, &number_xinerama_screens);
-
-//  g_print("number_xinerama_screens: %i\n", number_xinerama_screens);
-  if (screens) {
-//    g_print("xinerama screen: %i\n", user_screen);
-    if (user_screen > -1 && user_screen < number_xinerama_screens) {
-      // set user selected screen
-      *preferred_screen = screens[user_screen];
-//      g_print("preferred screen (user): %i\n", preferred_screen->screen_number);
-    } else {
-      // auto select largest screen
-      *preferred_screen
-        = *xinerama_maximize_screen_function(screens, number_xinerama_screens,
-                                             xinerama_screen_number_pixels);
-//      g_print("preferred screen (auto): %i\n", preferred_screen->screen_number);
+    if(i==0)
+    {
+      /* No icc markers found */
+      return NULL;
     }
-
-    // find lower rightmost screens for the statusbar
-    *statusbar_screen
-      = *xinerama_maximize_screen_function(screens, number_xinerama_screens,
-                                 xinerama_screen_lower_rightness);
-    XFree(screens);
+    else
+    {
+      /* Sort the markers and copy them together */
+      for(j=0; j< i; j++)
+      {
+        length+= tag_length[j];
+      }
+      icc_ptr = malloc(length+sizeof(length));
+      if(icc_ptr==NULL) return NULL;
+      *(unsigned int *)icc_ptr=length;
+      length=0;
+      for(j=0; j< i; j++)
+      {
+        memcpy(icc_ptr+length+sizeof(length), tag_ptr[j], tag_length[j]);
+        length+= tag_length[j];
+      }
+      free(tag_ptr);
+      free(tag_length);
+    }
+    jpeg_destroy_decompress(&cinfo);
+    return icc_ptr;
   }
-  else {
-    /* If we don't have Xinerama, fake it: */
-    preferred_screen->screen_number = 0;
-    preferred_screen->x_org = 0;
-    preferred_screen->y_org = 0;
-    preferred_screen->width = gdk_screen_width();
-    preferred_screen->height = gdk_screen_height();
+  /* is pic a tiff?*/
+  else if((pic_tst[0] == pic_tst[1]) && ((pic_tst[0] == 0x49)||(pic_tst[0] == 0x4d)) && (pic_tst[2] == 0x2a) &&  (pic_tst[3] == 0x00)) 
+  {
+    uint16 count;
+    unsigned char *data;
 
-    *statusbar_screen = *preferred_screen;
+    fclose(infile);
+    if((tiff_image = TIFFOpen(filename, "r")) == NULL){
+      fprintf(stderr, "Could not open incoming image\n");
+      return NULL;
+    }
+    if (TIFFGetField(tiff_image, TIFFTAG_ICCPROFILE, &count, &data))
+    {
+      length = count;
+      icc_ptr = malloc(length+sizeof(length));
+      *(unsigned int *)icc_ptr=length;
+      memcpy(icc_ptr+sizeof(length), data, length);
+    }
+    TIFFClose(tiff_image);
+    return icc_ptr;
   }
-  if (dpy)
-    XCloseDisplay(dpy);
+
+  fclose(infile);
+  return NULL;
 }
 #endif
+
+#ifdef HAVE_EXIF
+char **get_exif_values(char *filename)
+{
+  ExifData *ed;
+  ExifEntry *entry;
+  int i,j;
+  char *line;
+  char **exif_lines=NULL;
+  char buffer[256];
+
+  /* displayed exif tags, when available */
+  struct {
+    ExifTag tag;
+    ExifIfd ifd;
+  } tags[] = {
+    {EXIF_TAG_MAKE, EXIF_IFD_0},
+    {EXIF_TAG_MODEL, EXIF_IFD_0},
+    {EXIF_TAG_DOCUMENT_NAME, EXIF_IFD_0},
+    {EXIF_TAG_PIXEL_X_DIMENSION , EXIF_IFD_EXIF},
+    {EXIF_TAG_PIXEL_Y_DIMENSION , EXIF_IFD_EXIF},
+    {EXIF_TAG_SOFTWARE, EXIF_IFD_0},
+    {EXIF_TAG_DATE_TIME, EXIF_IFD_0},
+    {EXIF_TAG_DATE_TIME_ORIGINAL, EXIF_IFD_EXIF},
+    {EXIF_TAG_ARTIST, EXIF_IFD_0},
+    {EXIF_TAG_ORIENTATION, EXIF_IFD_0},
+    {EXIF_TAG_JPEG_PROC, EXIF_IFD_1},
+    {EXIF_TAG_FLASH, EXIF_IFD_EXIF},
+    {EXIF_TAG_EXPOSURE_TIME, EXIF_IFD_EXIF},
+    {EXIF_TAG_ISO_SPEED_RATINGS, EXIF_IFD_EXIF},
+    {EXIF_TAG_APERTURE_VALUE, EXIF_IFD_EXIF},
+    {EXIF_TAG_FOCAL_LENGTH, EXIF_IFD_EXIF},
+    {EXIF_TAG_FOCAL_LENGTH_IN_35MM_FILM, EXIF_IFD_EXIF},
+    {EXIF_TAG_SHUTTER_SPEED_VALUE, EXIF_IFD_EXIF},
+    {EXIF_TAG_SUBJECT_DISTANCE, EXIF_IFD_EXIF},
+    {EXIF_TAG_METERING_MODE, EXIF_IFD_EXIF},
+    {EXIF_TAG_EXPOSURE_MODE, EXIF_IFD_EXIF},
+    {EXIF_TAG_SENSING_METHOD, EXIF_IFD_EXIF},
+    {EXIF_TAG_WHITE_BALANCE, EXIF_IFD_EXIF},
+    {EXIF_TAG_GAIN_CONTROL, EXIF_IFD_EXIF},
+    {EXIF_TAG_COLOR_SPACE, EXIF_IFD_EXIF},
+    {EXIF_TAG_CONTRAST, EXIF_IFD_EXIF},
+    {EXIF_TAG_SATURATION, EXIF_IFD_EXIF},
+    {EXIF_TAG_SHARPNESS, EXIF_IFD_EXIF},
+    {EXIF_TAG_DIGITAL_ZOOM_RATIO, EXIF_IFD_EXIF}
+  };
+
+  ExifTag gps_tags[] = {
+    EXIF_TAG_GPS_LATITUDE, EXIF_TAG_GPS_LATITUDE_REF,
+    EXIF_TAG_GPS_LONGITUDE, EXIF_TAG_GPS_LONGITUDE_REF,
+    EXIF_TAG_GPS_ALTITUDE, EXIF_TAG_GPS_ALTITUDE
+  };
+
+  j=0;
+  ed = exif_data_new_from_file(filename);
+  if(ed)
+  {
+    /* one too much to make sure the last one will allways be NULL
+       size of tags + gpstags(3) + extra lines(3) + 1 */ 
+    exif_lines=calloc(sizeof(tags)/sizeof(tags[0]),sizeof(char*)+3+3+1);
+    for(i=0; i < sizeof(tags)/sizeof(tags[0]); i++)
+    {
+      entry=exif_content_get_entry( ed->ifd[tags[i].ifd], tags[i].tag);
+      if(entry)
+      {
+        line=malloc(256); 
+        exif_entry_get_value (entry, buffer, 255);
+        snprintf(line, 255, "%-21s: %s\n", exif_tag_get_name_in_ifd(tags[i].tag,tags[i].ifd), buffer);
+        exif_lines[j]=line;
+        j++;
+      }
+    }
+    for(i=0; i < 6; i+=2)
+    {
+      entry=exif_content_get_entry( ed->ifd[EXIF_IFD_GPS], gps_tags[i]);
+      if(entry)
+      {
+        line=malloc(256); 
+        exif_entry_get_value (entry, buffer, 255);
+        if((entry=exif_content_get_entry( ed->ifd[EXIF_IFD_GPS], i+1)))
+        {
+          int len=strlen(buffer);
+          if(len < 254)
+          {
+            buffer[len]=0x20;
+            exif_entry_get_value(entry, buffer+len+1, 254-len);
+          }
+        }
+        snprintf(line, 255, "%-21s: %s\n", exif_tag_get_name_in_ifd(gps_tags[i], EXIF_IFD_GPS), buffer);
+        exif_lines[j]=line;
+        j++;
+      }
+    }
+  }
+  if(j==0)
+  {
+    free(exif_lines);
+    return NULL;
+  }
+  line=malloc(64);
+  snprintf(line, 63, "%-21s: %i Bytes\n", "FileSize", (int)file_size);
+  exif_lines[j++]=line;
+  if(jpeg_prog)
+  {
+    line=malloc(64);
+    snprintf(line, 63, "%-21s: %s\n", "JpegProcess", "Progressive");
+    exif_lines[j++]=line;
+  }
+  if(comment)
+  {
+    line=malloc(256);
+    snprintf(line, 255, "%-21s: %s\n", "Comment", comment);
+    exif_lines[j++]=line;
+  }
+    
+  return exif_lines;
+}
+#endif
+
+/* The state of DPMS is checked at startup of qiv.
+   If the display is DPMS capable and DPMS is activated, qiv disables
+   it during a slideshow to prevent the screen from blanking. */
+BOOL dpms_enabled = 0;
+
+void dpms_check()
+{
+  int not_needed;
+  Display * disp = GDK_DISPLAY();
+  CARD16 power_level;
+
+  if(DPMSQueryExtension(disp, &not_needed, &not_needed))
+  { 
+    if(DPMSCapable(disp))
+    {
+      DPMSInfo(disp, &power_level, &dpms_enabled);
+    }
+  }
+}
+
+void dpms_enable()
+{
+  if(dpms_enabled)
+  { 
+    DPMSEnable(GDK_DISPLAY());
+  }
+}
+
+void dpms_disable()
+{
+  if(dpms_enabled)
+  { 
+    DPMSDisable(GDK_DISPLAY());
+  }
+}
